
/***************************************************************************
*
*   Copyright (c) 1998, 1999 Timpanogas Research Group, Inc.
*   895 West Center Street
*   Orem, Utah  84057
*   jmerkey@timpanogas.com
*
*   This program is free software; you can redistribute it and/or modify it
*   under the terms of the GNU General Public License as published by the
*   Free Software Foundation, version 2, or any later version.
*
*   This program is distributed in the hope that it will be useful, but
*   WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   General Public License for more details.
*
*   You are free to modify and re-distribute this program in accordance
*   with the terms specified in the GNU Public License.  The copyright
*   contained in this code is required to be present in any derivative
*   works and you are required to provide the source code for this
*   program as part of any commercial or non-commercial distribution.
*   You are required to respect the rights of the Copyright holders
*   named within this code.
*
*   jmerkey@timpanogas.com and TRG, Inc. are the official maintainers of
*   this code.  You are encouraged to report any bugs, problems, fixes,
*   suggestions, and comments about this software to jmerkey@timpanogas.com
*   or linux-kernel@vger.rutgers.edu.  New releases, patches, bug fixes, and
*   technical documentation can be found at www.timpanogas.com.  TRG will
*   periodically post new releases of this software to www.timpanogas.com
*   that contain bug fixes and enhanced capabilities.
*
*   Original Authorship      :
*      source code written by Jeff V. Merkey, TRG, Inc.
*
*   Original Contributors    :
*      Jeff V. Merkey, TRG, Inc.
*      Darren Major, TRG, Inc.
*      Alan Cox, RedHat Software, Inc.
*
****************************************************************************
*
*   AUTHOR   :  Jeff V. Merkey (jmerkey@timpanogas.com)
*   FILE     :  GLOBALS.H
*   DESCRIP  :  FENRIS Global Declarations
*   DATE     :  November 1, 1998
*
*
***************************************************************************/

#ifndef _NWFS_GLOBALS_
#define _NWFS_GLOBALS_

//
//   select target
//

#define  WINDOWS_NT_RO    0
#define  WINDOWS_NT       0
#define  WINDOWS_NT_UTIL  0
#define  WINDOWS_CONVERT  0
#define  WINDOWS_98_UTIL  0
#define  LINUX_20         0
#define  LINUX_22         1
#define  LINUX_24         0
#define  LINUX_UTIL       0
#define  DOS_UTIL         0

#if (LINUX_20)
#define LINUX            1
#define LINUX_SLEEP      1
#define LINUX_SPIN       0
#endif
#if (LINUX_22)
#define LINUX            1
#define LINUX_SLEEP      1
#define LINUX_SPIN       1
#endif
#if (LINUX_24)
#define LINUX            1
#define LINUX_SLEEP      1
#define LINUX_SPIN       1
#endif
#if (LINUX_UTIL)
#define LINUX            1
#endif

#if (WINDOWS_98_UTIL)
#undef DOS_UTIL
#define DOS_UTIL         1
#endif

//
//   define version of this utility
//

#define  MAJOR_VERSION  2
#define  MINOR_VERSION  2
#define  BUILD_VERSION  3

// debug flags
#define  HOTFIX_VERBOSE         0  // trace hotfix table lookups
#define  VOLUME_DUMP            0  // dump volume segment info as detected
#define  DEBUG_SA               0  // trace suballocation reads/writes
#define  CHECK_NLINK            0  // perform nlink validation during dir delete
#define  DEBUG_DEADLOCKS        0  // allows deadlocks to receive EINTR signals
#define  VFS_DEBUG              0  // debug vfs interface calls
#define  PROFILE_AIO            0  // profile aio efficiency
#define  DEBUG_AIO              0  // trace linux AIO calls
#define  POST_IMMEDIATE         1  // force immediate post of I/O requests 
#define  DUMP_BUFFER_HEADS      0  // dump buffer heads to /var/log/messages

// feature enable flags
#define  VERBOSE                0  // enable verbose output
#define  MOUNT_VERBOSE          0  // enable verbose output during mounts
#define  READ_AHEAD_ON          1  // enable read ahead capability
#define  ASYNCH_READ_AHEAD      1  // enable asynch read ahead capability
#define  SALVAGEABLE_FS         0  // enable the salvageable file system
#define  STRICT_NAMES           1  // strict name validation during mount
#define  LRU_FACTOR             2  // % dirty for flush (1-100%/2-50%/3-33%)
#define  LRU_AGE                2  // how many seconds to age dirty blocks
#define  ALLOW_REMOUNT          1  // allow remounting of NetWare Volumes

// LEAVE DISABLED FOR NOW!!! BUG
#define  PAGE_CACHE_ON          0  // enable Linux 2.4 page cache interface

// NOTE:  if you disable this, then the elevator moves to the LRU
// otherwise the AIO manager maintains the elevator.
#define  DO_ASYNCH_IO           1  // enable the asynch I/O subsystem
#define  LINUX_BUFFER_CACHE     0  // double buffer data in the linux cache
#define  ZERO_FILL_SECTORS      0  // zero fill newly allocated sectors
#define  READER_WRITER_LOCKS    0  // enable SMP reader/writer locks
#define  CONSERVE_MEMORY        0  // conserve unused AIO/WorkSpace Memory
#define  CONSERVE_HASH_MEMORY   1  // conserve unused hash memory
#define  LINUX_AIO              1  // use Linux 2.2/2.4 AIO subsystem
#define  HASH_FAT_CHAINS        1  // hash file FAT chain heads 

// these values can be changed if you want to increase the numbers of
// disks/volumes that NWFS will support

#define MAX_DISKS             325  // max disks
#define MAX_PARTITIONS        MAX_DISKS * 4
#define MAX_VOLUMES           256  // netware supports 256 volumes/server
#define MAX_BUFFER_HEADS      256  // buffer head pool for AIO
#define MAX_WORKSPACE_BUFFERS  32  // max workspaces/volume

// these values are architecture dependent and should not be changed
#define MAX_VOLUME_ENTRIES      8  // 8 entries per volume table (partition)
#define MAX_SEGMENTS           32  // a volume can have 32 segments
#define MAX_MIRRORS             8  // netware max is 8
#define MAX_NAMESPACES         10  // 12 in 4.x, 10 in 3.x
#define MAX_SUBALLOC_NODES      5  // max number of suballoc headers/volume
#define MAX_SUBALLOC_SPAN       2  // max suballoc nodes an entry can span
#define SUBALLOC_BLOCK_SIZE   512  // minumum suballocation size
#define MAX_HOTFIX_BLOCKS      30  // max hotfix block list table entries 

#define IO_BLOCK_SIZE        4096  // volume block size default
#define BIT_BLOCK_SIZE       IO_BLOCK_SIZE
#define HOTFIX_BLOCK_SIZE    IO_BLOCK_SIZE

// this defines the maximum size of the SMP LRU for NWFS.  If you
// start getting "virtual memory exhausted" messages, you may want
// to crank down this value for Linux.  Default is 2000 buffers

#define MIN_FAT_LRU_BLOCKS        0
#define MAX_FAT_LRU_BLOCKS        100

#define MIN_DIRECTORY_LRU_BLOCKS  10
#define MAX_DIRECTORY_LRU_BLOCKS  500

#define MIN_VOLUME_LRU_BLOCKS     10
#define MAX_VOLUME_LRU_BLOCKS     1000

#if (DOS_UTIL)
#undef  MAX_VOLUME_LRU_BLOCKS
#undef  MAX_DIRECTORY_LRU_BLOCKS
#define MAX_VOLUME_LRU_BLOCKS     500
#define MAX_DIRECTORY_LRU_BLOCKS  500
#endif

#if (WINDOWS_NT | WINDOWS_NT_RO)
#undef  MAX_VOLUME_LRU_BLOCKS
#undef  MAX_DIRECTORY_LRU_BLOCKS
#define MAX_VOLUME_LRU_BLOCKS     2000
#define MAX_DIRECTORY_LRU_BLOCKS  1000
#endif

#if (LINUX_20 | LINUX_22)
#undef  MAX_VOLUME_LRU_BLOCKS
#undef  MAX_DIRECTORY_LRU_BLOCKS
#define MAX_VOLUME_LRU_BLOCKS     2000
#define MAX_DIRECTORY_LRU_BLOCKS  1000
#endif

#if (LINUX_24)
#undef  MAX_VOLUME_LRU_BLOCKS
#undef  MAX_DIRECTORY_LRU_BLOCKS
#if (LINUX_BUFFER_CACHE)
#define MAX_VOLUME_LRU_BLOCKS     1000
#define MAX_DIRECTORY_LRU_BLOCKS  1000
#else
#define MAX_VOLUME_LRU_BLOCKS     4000
#define MAX_DIRECTORY_LRU_BLOCKS  1000
#endif
#endif

// LRU Eviction priority values
#define DATA_PRIORITY           1
#define DIRECTORY_PRIORITY      2
#define FAT_PRIORITY            3

//
//  global includes for NT
//

#if (WINDOWS_NT | WINDOWS_NT_RO)

#define TargetOS "Windows NT"

#include "ntifs.h"
#include "ntdddisk.h"

#define NWFSPrint DbgPrint

typedef UCHAR BYTE;
typedef USHORT WORD;

#include "stdio.h"
#include "stdlib.h"

#include "nwfs.h"
#include "nwhash.h"
#include "nwstruct.h"
#include "nwerror.h"
#include "nwdir.h"
#include "nwproc.h"

#include "nwvphal.h"
#include "nwvp.h"

#include "nwioctl.h"

#endif

//
//  global includes for NT utilities
//

#if (WINDOWS_NT_UTIL)

#define CONFIG_NAME        "  NWFS Volume Manager for Windows NT/2000"
#define IMAGER_NAME        "  NWFS Volume IMAGER for Windows NT/2000"
#define TargetOS "Windows NT"

#define COPYRIGHT_NOTICE1  "  Copyright (c) 1999, 2000 TRG, Inc.  All Rights Reserved."
#define COPYRIGHT_NOTICE2  "  Portions Copyright (c) Lintell, Inc."


#include "windows.h"
#include "winioctl.h"
#include "winuser.h"
#include "stdarg.h"

extern int log_printf(char *format, ...);
extern int log_sprintf(char *buffer, char *format, ...);

#define NWFSPrint log_printf

typedef UCHAR BYTE;
typedef USHORT WORD;

#include "stdio.h"
#include "stdlib.h"
#include "ctype.h"
#include "conio.h"

#include "nwfs.h"
#include "nwhash.h"
#include "nwstruct.h"
#include "nwerror.h"
#include "nwdir.h"
#include "nwproc.h"

#include "nwvphal.h"
#include "nwvp.h"
#include "fileutil.h"

#include "nwmenu.h"

#endif

//
//
//

#if (DOS_UTIL)

#if (WINDOWS_98_UTIL)
#define CONFIG_NAME        "  NWFS Volume Manager for Windows 95/98"
#define IMAGER_NAME        "  NWFS Volume IMAGER for Windows 95/98"
#define TargetOS "Windows 95/98"
#else
#define CONFIG_NAME        "  NWFS Volume Manager for DOS"
#define IMAGER_NAME        "  NWFS Volume IMAGER for DOS"
#define TargetOS "DOS"
#endif

#define COPYRIGHT_NOTICE1  "  Copyright (c) 1999, 2000 TRG, Inc.  All Rights Reserved."
#define COPYRIGHT_NOTICE2  "  Portions Copyright (c) Lintell, Inc."

#define  TRUE               1
#define  FALSE              0

#include "unistd.h"
#include "stdarg.h"
#include "stdio.h"
#include "stdlib.h"
#include "fcntl.h"
#include "ctype.h"
#include "string.h"
#include "memory.h"
#include "dos.h"
#include "conio.h"

#include "sys/farptr.h"
#include "sys/movedata.h"
#include "sys/segments.h"
#include "sys/stat.h"
#include "dpmi.h"

#include "time.h"

extern int log_printf(char *format, ...);
extern int log_sprintf(char *buffer, char *format, ...);

#define NWFSPrint log_printf

#ifndef LONGLONG
typedef long long LONGLONG;
#endif

typedef unsigned long  ULONG;
typedef unsigned short WORD;
typedef unsigned char  BYTE;

#include "nwfs.h"
#include "nwhash.h"
#include "nwstruct.h"
#include "nwerror.h"
#include "nwdir.h"
#include "nwproc.h"

#include "nwvphal.h"
#include "nwvp.h"
#include "fileutil.h"

#include "nwmenu.h"

#endif

//
//  global includes for Linux
//

//
//   Linux version 2.0
//

#if (LINUX_20)

#define TargetOS "Linux 20"

#define  TRUE               1
#define  FALSE              0

#ifndef LONGLONG
typedef long long LONGLONG;
#endif

typedef unsigned long  ULONG;
typedef unsigned short WORD;
typedef unsigned char  BYTE;

#define NWFSPrint printk

#if MODULE
#ifdef NWFSMOD_VERSION_INFO
#include <linux/module.h>
#endif
#if MODVERSIONS
#include <linux/modversions.h>
#endif
#endif

#include <linux/version.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <linux/fcntl.h>
#include <linux/malloc.h>
#include <linux/genhd.h>
#include <linux/hdreg.h>
#include <linux/fs.h>
#include <linux/major.h>
#include <linux/mm.h>
#include <linux/cdrom.h>
#include <linux/locks.h>
#include <linux/pagemap.h>
#include <linux/swap.h>
#include <linux/swapctl.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>

#include <asm/system.h>
#include <asm/segment.h>
#include <asm/atomic.h>
#include <sys/ioctl.h>

#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>

#define NWFSInitMutex(s)        struct semaphore (s)=MUTEX
#define NWFSInitSemaphore(s)    struct semaphore (s)=MUTEX_LOCKED

#include "nwfs.h"
#include "nwhash.h"
#include "nwstruct.h"
#include "nwerror.h"
#include "nwdir.h"
#include "nwproc.h"

#include "nwvphal.h"
#include "nwvp.h"

#include "nwvfs.h"

#endif

//
//   Linux version 2.2 
//

#if (LINUX_22)

#define TargetOS "Linux 22"

#define  TRUE               1
#define  FALSE              0

#ifndef LONGLONG
typedef long long LONGLONG;
#endif

typedef unsigned long  ULONG;
typedef unsigned short WORD;
typedef unsigned char  BYTE;

#define NWFSPrint printk

#if MODULE
#ifndef NWFSMOD_VERSION_INFO
#define __NO_VERSION__
#endif
#if MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#endif

#include <linux/version.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <linux/fcntl.h>
#include <linux/malloc.h>
#include <linux/vmalloc.h>
#include <linux/genhd.h>
#include <linux/hdreg.h>
#include <linux/fs.h>
#include <linux/major.h>
#include <linux/mm.h>
#include <linux/cdrom.h>
#include <linux/locks.h>
#include <linux/pagemap.h>
#include <linux/swap.h>
#include <linux/swapctl.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/segment.h>
#include <asm/atomic.h>

#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>

#define NWFSInitMutex(s)        struct semaphore (s)=MUTEX
#define NWFSInitSemaphore(s)    struct semaphore (s)=MUTEX_LOCKED

#include "nwfs.h"
#include "nwhash.h"
#include "nwstruct.h"
#include "nwerror.h"
#include "nwdir.h"
#include "nwproc.h"

#include "nwvphal.h"
#include "nwvp.h"

#include "nwvfs.h"

#endif

//
//
//   Linux version 2.4 
//

#if (LINUX_24)

#define TargetOS "Linux 24"

#define NWFSInitMutex(s)         DECLARE_MUTEX(s)
#define NWFSInitSemaphore(s)     DECLARE_MUTEX_LOCKED(s)

#define  TRUE               1
#define  FALSE              0

#ifndef LONGLONG
typedef long long LONGLONG;
#endif

typedef unsigned long  ULONG;
typedef unsigned short WORD;
typedef unsigned char  BYTE;

#define NWFSPrint printk

#if MODULE
#ifndef NWFSMOD_VERSION_INFO
#define __NO_VERSION__
#endif
#if MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#endif

#include <linux/version.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <linux/fcntl.h>
#include <linux/malloc.h>
#include <linux/vmalloc.h>
#include <linux/genhd.h>
#include <linux/hdreg.h>
#include <linux/fs.h>
#include <linux/major.h>
#include <linux/mm.h>
#include <linux/cdrom.h>
#include <linux/locks.h>
#include <linux/pagemap.h>
#include <linux/swap.h>
#include <linux/swapctl.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/segment.h>
#include <asm/atomic.h>

#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>

#include "nwfs.h"
#include "nwhash.h"
#include "nwstruct.h"
#include "nwerror.h"
#include "nwdir.h"
#include "nwproc.h"

#include "nwvphal.h"
#include "nwvp.h"

#include "nwvfs.h"

#endif

//
//
//

#if (LINUX_UTIL)

#define CONFIG_NAME        "  NWFS Volume Manager for Linux"
#define IMAGER_NAME        "  NWFS Volume IMAGER for Linux"
#define COPYRIGHT_NOTICE1  "  Copyright (c) 1999, 2000 TRG, Inc.  All Rights Reserved."
#define COPYRIGHT_NOTICE2  "  Portions Copyright (c) Lintell, Inc."

#define TargetOS "Linux"

extern int log_printf(char *format, ...);
extern int log_sprintf(char *buffer, char *format, ...);

#define NWFSPrint log_printf

#define VCS_RESERVED   4

#include "unistd.h"
#include "stdio.h"
#include "stdlib.h"
#include "fcntl.h"
#include "ctype.h"
#include "time.h"
#include "linux/string.h"
#include "ncurses.h"
#include "termios.h"

#ifndef LONGLONG
typedef long long LONGLONG;
#endif

typedef unsigned long  ULONG;
typedef unsigned short WORD;
typedef unsigned char  BYTE;

#include <sys/ioctl.h>
#include <sys/stat.h>
#include <linux/genhd.h>
#include <linux/hdreg.h>
#include <linux/fs.h>
#include <linux/errno.h>

#include "nwfs.h"
#include "nwhash.h"
#include "nwstruct.h"
#include "nwerror.h"
#include "nwdir.h"
#include "nwproc.h"

#include "nwvphal.h"
#include "nwvp.h"

#include "nwmenu.h"

#endif

#endif     //  _NWFS_GLOBALS_
